// DockStation.h : Declaration of the CDockStation

#ifndef __DOCKSTATION_H_
#define __DOCKSTATION_H_

#include "resource.h"       // main symbols
#include <atlctl.h>
#include "msgcom.h"
#include "hostex.h"
#include "Popupmgr.h"

// Read only stream from resource
struct CStreamOnResource
{
	CStreamOnResource(int resid, LPCTSTR strType)
	{
		HRSRC hrc = FindResource(_Module.GetModuleInstance(), MAKEINTRESOURCE(resid), strType);
 		HGLOBAL h = ::LoadResource(_Module.GetModuleInstance(), hrc);
		pMem = GlobalAlloc(GMEM_FIXED, SizeofResource(_Module.GetModuleInstance(), hrc));
		::CopyMemory(pMem, h, SizeofResource(_Module.GetModuleInstance(), hrc));
 
		pStm = NULL;
		HRESULT hr = ::CreateStreamOnHGlobal(pMem, false, &pStm);
		
	}

	~CStreamOnResource()
	{
		pStm->Release();
		GlobalFree(pMem);
	}

	IStream* GetStream() { return pStm;	}

	PVOID pMem;
	IStream* pStm;
};


/////////////////////////////////////////////////////////////////////////////
// CDockStation
class ATL_NO_VTABLE CDockStation : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public IDockStation,
	public IListener,
	public CComControl<CDockStation>,
	public IPersistStreamInitImpl<CDockStation>,
	public IOleControlImpl<CDockStation>,
	public IOleObjectImpl<CDockStation>,
	public IOleInPlaceActiveObjectImpl<CDockStation>,
	public IViewObjectExImpl<CDockStation>,
	public IOleInPlaceObjectWindowlessImpl<CDockStation>,
	public IPersistStorageImpl<CDockStation>,
	public ISpecifyPropertyPagesImpl<CDockStation>,
	public IQuickActivateImpl<CDockStation>,
	public IDataObjectImpl<CDockStation>,
	public IProvideClassInfo2Impl<&CLSID_DockStation, NULL, &LIBID_CONT2Lib>,
	public CComCoClass<CDockStation, &CLSID_DockStation>
{
public:
	CDockStation()
	{
		m_bWindowOnly = TRUE;

		SIZE ext = { 320, 280 };
		AtlPixelToHiMetric( &ext, &ext );
		m_sizeExtent = m_sizeNatural = ext;
	}

DECLARE_REGISTRY_RESOURCEID(IDR_DOCKSTATION)
DECLARE_NOT_AGGREGATABLE(CDockStation)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CDockStation)
	COM_INTERFACE_ENTRY(IDockStation)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
	COM_INTERFACE_ENTRY(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY(IQuickActivate)
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
	COM_INTERFACE_ENTRY(IListener)
END_COM_MAP()

BEGIN_PROP_MAP(CDockStation)
	PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
END_PROP_MAP()

BEGIN_MSG_MAP(CDockStation)
	MESSAGE_HANDLER(WM_CREATE, OnCreate)
	MESSAGE_HANDLER(WM_SIZE, OnSize)
	CHAIN_MSG_MAP(CComControl<CDockStation>)
	DEFAULT_REFLECTION_HANDLER()
END_MSG_MAP()
// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);



// IViewObjectEx
	DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

	LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		AtlAxWinInit();
		RECT rc = {0,0,0,26};
		//"cont2.containex"
		USES_CONVERSION;
		CStreamOnResource rstm(IDR_MENU,"CUSTOM");

		m_hToolWnd.Create(m_hWnd, rc, "", WS_CHILD | WS_VISIBLE);
		AtlAxCreateControl2(T2COLE("Contain.CTool"), m_hToolWnd, NULL, NULL);

		m_hMenuWnd.Create(m_hWnd, rc, "", WS_CHILD | WS_VISIBLE);
		AtlAxCreateControl2(T2COLE("Contain.CTool"), m_hMenuWnd, rstm.GetStream(), NULL);
		ATLASSERT(m_hToolWnd);

		rc.top = 26;
		m_hOGLWnd.Create(m_hWnd, rc, "{72D0889C-0EBD-11D0-BF0D-0000E8D0D146}", WS_CHILD | WS_VISIBLE);

		ModifyStyleEx(0, WS_EX_STATICEDGE, 0);
		
		IUnknown* pUnk;
		//HRESULT hr = ::CoCreateInstance(CLSID_MsgCom, NULL, CLSCTX_INPROC_SERVER, 
		//					IID_IUnknown, (void**)&pUnk);
		
		IClassFactory* pCF;
		_Module.GetClassObject(CLSID_MsgCom, IID_IClassFactory, (void**)&pCF);
		HRESULT hr = pCF->CreateInstance(NULL, IID_IMsgCom, (void**)&m_pMsgMan);

		IListener* pListener;
		QueryInterface(IID_IListener, (void**)&pListener);
		m_pMsgMan->RegisterListener(pListener);

		return 0;
	}

	LRESULT OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		int cx = LOWORD(lParam);
		int cy = HIWORD(lParam);

		::SetWindowPos(m_hMenuWnd, NULL, 0, 0, cx, 23, SWP_NOZORDER | SWP_NOMOVE);
		::SetWindowPos(m_hToolWnd, NULL, 0, 23, cx, 26, SWP_NOZORDER);
		::SetWindowPos(m_hOGLWnd, NULL, 0, 49, cx, cy - 49, SWP_NOZORDER);
		return 0;
	}

// IDockStation
public:
//	BEGIN_HANDLER_MAP == HandleMessage()
//		ON_HANDLE(IDx, DoCaboom)
//	END_HANDLER_MAP
	STDMETHOD(HandleMessage)(IMessage* pMsg)
	{
	//	IMessage* pMsg;
	//	if (S_OK == QueryInterface(IID_IMessage, (void**)&pMsg))
		{
			int id;
			pMsg->GetID(&id);
			switch (id)
			{
			case 1:
				MessageBox("Ca-Boom....", "Command", MB_ICONASTERISK | MB_OK);
				break;
			case 2:
				::MessageBox(m_hWnd, "E=MC2", "Command", MB_ICONASTERISK | MB_OK);
				break;
			case 103:
				MessageBox("Made with pure ATL.\nDenis Bekman (c)1999", "About", MB_ICONASTERISK | MB_OK);
				break;

			}
		}
		; //if GUID == "xxx-x" -> do ogl->Explode
		return S_OK;
	}

	HRESULT OnDraw(ATL_DRAWINFO& di)
	{
		/*RECT& rc = *(RECT*)di.prcBounds;
		Rectangle(di.hdcDraw, rc.left, rc.top, rc.right, rc.bottom);

		SetTextAlign(di.hdcDraw, TA_CENTER|TA_BASELINE);
		LPCTSTR pszText = _T("ATL 3.0 : DockStation");
		TextOut(di.hdcDraw, 
			(rc.left + rc.right) / 2, 
			(rc.top + rc.bottom) / 2, 
			pszText, 
			lstrlen(pszText));
*/
		return S_OK;
	}

private:
	//CWindowImpl	m_hToolWnd;
	CAxWindow	m_hMenuWnd;
	CAxWindow	m_hToolWnd;
	CAxWindow	m_hOGLWnd;

	IMsgCom*	m_pMsgMan;
//	CComObject<CPopupMgr>	m_popMgr;
};

#endif //__DOCKSTATION_H_
